@@ -12,7 +12,7 @@ Lightweight Event driven Framework
 """
 
 __author__ = "James Mills"
-__date__ = "25 January 2011"
-__version__ = "1.3"
+__date__ = "1st February 2011"
+__version__ = "1.3.2"
 
 from core import *
