@@ -10,19 +10,20 @@ These tools can also be used within Controlelrs and request handlers.
 
 import os
 import stat
-import types
 import hashlib
-import datetime
 import mimetypes
 import mimetools
+from time import mktime
 from rfc822 import formatdate
+from datetime import datetime, timedelta
 
 mimetypes.init()
-mimetypes.types_map['.dwg']='image/x-dwg'
-mimetypes.types_map['.ico']='image/x-icon'
+mimetypes.add_type("image/x-dwg", ".dwg")
+mimetypes.add_type("image/x-icon", ".ico")
+mimetypes.add_type("application/xhtml+xml", ".xhtml")
 
 import _httpauth
-from utils import url, valid_status, get_ranges
+from utils import get_ranges, compress
 from errors import HTTPError, NotFound, Redirect, Unauthorized
 
 def expires(request, response, secs=0, force=False):
@@ -53,7 +54,7 @@ def expires(request, response, secs=0, force=False):
                 break
     
     if not cacheable:
-        if isinstance(secs, datetime.timedelta):
+        if isinstance(secs, timedelta):
             secs = (86400 * secs.days) + secs.seconds
         
         if secs == 0:
@@ -63,7 +64,9 @@ def expires(request, response, secs=0, force=False):
                 if force or "Cache-Control" not in headers:
                     headers["Cache-Control"] = "no-cache, must-revalidate"
             # Set an explicit Expires date in the past.
-            expiry = formatdate(1169942400.0)
+            now = datetime.now()
+            lastyear = now.replace(year=now.year-1)
+            expiry = formatdate(mktime(lastyear.timetuple()))
         else:
             expiry = formatdate(response.time + secs)
         if force or "Expires" not in headers:
@@ -127,14 +130,13 @@ def serve_file(request, response, path, type=None, disposition=None, name=None):
         r = get_ranges(request.headers.get('Range'), c_len)
         if r == []:
             response.headers['Content-Range'] = "bytes */%s" % c_len
-            message = "Invalid Range (first-byte-pos greater than Content-Length)"
-            return HTTPError(request, response, 416, message)
+            return HTTPError(request, response, 416)
         if r:
             if len(r) == 1:
                 # Return a single-part response.
                 start, stop = r[0]
                 r_len = stop - start
-                response.status = "206 Partial Content"
+                response.code = 206
                 response.headers['Content-Range'] = ("bytes %s-%s/%s" %
                                                        (start, stop - 1, c_len))
                 response.headers['Content-Length'] = r_len
@@ -142,7 +144,7 @@ def serve_file(request, response, path, type=None, disposition=None, name=None):
                 response.body = bodyfile.read(r_len)
             else:
                 # Return a multipart/byteranges response.
-                response.status = "206 Partial Content"
+                response.code = 206
                 boundary = mimetools.choose_boundary()
                 ct = "multipart/byteranges; boundary=%s" % boundary
                 response.headers['Content-Type'] = ct
@@ -206,13 +208,13 @@ def validate_etags(request, response, autotags=False):
     if hasattr(response, "ETag"):
         return
     
-    status, reason, msg = valid_status(response.status)
+    code = response.code
     
     etag = response.headers.get('ETag')
     
     # Automatic ETag generation. See warning in docstring.
     if (not etag) and autotags:
-        if status == 200:
+        if code == 200:
             etag = response.collapse_body()
             etag = '"%s"' % hashlib.md5.new(etag).hexdigest()
             response.headers['ETag'] = etag
@@ -222,23 +224,24 @@ def validate_etags(request, response, autotags=False):
     # "If the request would, without the If-Match header field, result in
     # anything other than a 2xx or 412 status, then the If-Match header
     # MUST be ignored."
-    if status >= 200 and status <= 299:
+    if code >= 200 and code <= 299:
         conditions = request.headers.elements('If-Match') or []
         conditions = [str(x) for x in conditions]
         if conditions and not (conditions == ["*"] or etag in conditions):
             return HTTPError(request, response, 412,
-                    "If-Match failed: ETag %r did not match %r" % (
+                    description="If-Match failed: ETag %r did not match %r" % (
                         etag, conditions))
         
         conditions = request.headers.elements('If-None-Match') or []
         conditions = [str(x) for x in conditions]
         if conditions == ["*"] or etag in conditions:
             if request.method in ("GET", "HEAD"):
-                return Redirect(request, response, [], 304)
+                return Redirect(request, response, [], code=304)
             else:
                 return HTTPError(request, response, 412,
-                        "If-None-Match failed: ETag %r matched %r" % (
-                            etag, conditions))
+                        description=(
+                            "If-None-Match failed: ETag %r matched %r" % (
+                                etag, conditions)))
 
 def validate_since(request, response):
     """Validate the current Last-Modified against If-Modified-Since headers.
@@ -249,75 +252,36 @@ def validate_since(request, response):
 
     lastmod = response.headers.get('Last-Modified')
     if lastmod:
-        status, reason, msg = valid_status(response.status)
+        code = response.code
         
         since = request.headers.get('If-Unmodified-Since')
         if since and since != lastmod:
-            if (status >= 200 and status <= 299) or status == 412:
+            if (code >= 200 and code <= 299) or code == 412:
                 return HTTPError(request, response, 412)
         
         since = request.headers.get('If-Modified-Since')
         if since and since == lastmod:
-            if (status >= 200 and status <= 299) or status == 304:
+            if (code >= 200 and code <= 299) or code == 304:
                 if request.method in ("GET", "HEAD"):
-                    return Redirect(request, response, [], 304)
+                    return Redirect(request, response, [], code=304)
                 else:
                     return HTTPError(request, response, 412)
 
-def redirect(request, response, url=''):
-    raise Redirect(request, response, url)
-
-def trailing_slash(request, response, missing=True, extra=False):
-    """Redirect if request.path has (missing|extra) trailing slash."""
-
-    pi = request.path
-    
-    if request.index is True:
-        if missing:
-            if not pi.endswith('/'):
-                new_url = url(request, pi + '/', request.qs)
-                return Redirect(request, response, new_url)
-    elif request.index is False:
-        if extra:
-            # If pi == '/', don't redirect to ''!
-            if pi.endswith('/') and pi != '/':
-                new_url = url(request, pi[:-1], request.qs)
-                return Redirect(request, response, new_url)
-    else:
-        return response
-
-def flatten(request, response):
-    """Wrap response.body in a generator that recursively iterates over body.
-    
-    This allows response.body to consist of 'nested generators';
-    that is, a set of generators that yield generators.
-    """
-    def flattener(input):
-        for x in input:
-            if not isinstance(x, types.GeneratorType):
-                yield x
-            else:
-                for y in flattener(x):
-                    yield y 
-
-    response.body = flattener(response.body)
-    return response
-
-def check_auth(request, response, users, encrypt=None, realm=None):
+def check_auth(request, response, realm, users, encrypt=None):
     """Check Authentication
 
     If an authorization header contains credentials, return True, else False.
 
-    @param realm: The authentication realm.
-    @type  realm: str
+    :param realm: The authentication realm.
+    :type  realm: str
 
-    @param users: A dict of the form: {username: password} or a callable
+    :param users: A dict of the form: {username: password} or a callable
                   returning a dict.
-    @type  users: dict or callable
+    :type  users: dict or callable
 
-    @param encrypt: Callable used to encrypt the password returned from
+    :param encrypt: Callable used to encrypt the password returned from
                     the user-agent. if None it defaults to a md5 encryption.
-    @type  encrypt: callable
+    :type  encrypt: callable
     """
 
     if 'authorization' in request.headers:
@@ -365,19 +329,19 @@ def basic_auth(request, response, realm, users, encrypt=None):
     If auth fails, returns an Unauthorized error  with a
     basic authentication header.
     
-    @param realm: The authentication realm.
-    @type  realm: str
+    :param realm: The authentication realm.
+    :type  realm: str
 
-    @param users: A dict of the form: {username: password} or a callable
+    :param users: A dict of the form: {username: password} or a callable
                   returning a dict.
-    @type  users: dict or callable
+    :type  users: dict or callable
 
-    @param encrypt: Callable used to encrypt the password returned from
+    :param encrypt: Callable used to encrypt the password returned from
                     the user-agent. if None it defaults to a md5 encryption.
-    @type  encrypt: callable
+    :type  encrypt: callable
     """
 
-    if check_auth(request, response, users, encrypt):
+    if check_auth(request, response, realm, users, encrypt):
         return
     
     # inform the user-agent this path is protected
@@ -390,18 +354,75 @@ def digest_auth(request, response, realm, users):
     
     If auth fails, raise 401 with a digest authentication header.
     
-    @param realm: The authentication realm.
-    @type  realm: str
+    :param realm: The authentication realm.
+    :type  realm: str
 
-    @param users: A dict of the form: {username: password} or a callable
+    :param users: A dict of the form: {username: password} or a callable
                   returning a dict.
-    @type  users: dict or callable
+    :type  users: dict or callable
     """
 
-    if check_auth(request, response, users, realm=realm):
+    if check_auth(request, response, realm, users):
         return
     
     # inform the user-agent this path is protected
     response.headers["WWW-Authenticate"] = _httpauth.digestAuth(realm)
     
     return Unauthorized(request, response)
+
+def gzip(response, level=4, mime_types=['text/html', 'text/plain']):
+    """Try to gzip the response body if Content-Type in mime_types.
+    
+    response.headers['Content-Type'] must be set to one of the
+    values in the mime_types arg before calling this function.
+    
+    No compression is performed if any of the following hold:
+        * The client sends no Accept-Encoding request header
+        * No 'gzip' or 'x-gzip' is present in the Accept-Encoding header
+        * No 'gzip' or 'x-gzip' with a qvalue > 0 is present
+        * The 'identity' value is given with a qvalue > 0.
+    """
+
+    if not response.body:
+        # Response body is empty (might be a 304 for instance)
+        return response
+    
+    # If returning cached content (which should already have been gzipped),
+    # don't re-zip.
+    if getattr(response.request, "cached", False):
+        return response
+    
+    acceptable = response.request.headers.elements('Accept-Encoding')
+    if not acceptable:
+        # If no Accept-Encoding field is present in a request,
+        # the server MAY assume that the client will accept any
+        # content coding. In this case, if "identity" is one of
+        # the available content-codings, then the server SHOULD use
+        # the "identity" content-coding, unless it has additional
+        # information that a different content-coding is meaningful
+        # to the client.
+        return response
+    
+    ct = response.headers.get('Content-Type', 'text/html').split(';')[0]
+    for coding in acceptable:
+        if coding.value == 'identity' and coding.qvalue != 0:
+            return response
+        if coding.value in ('gzip', 'x-gzip'):
+            if coding.qvalue == 0:
+                return response
+            if ct in mime_types:
+                # Return a generator that compresses the page
+                varies = response.headers.get("Vary", "")
+                varies = [x.strip() for x in varies.split(",") if x.strip()]
+                if "Accept-Encoding" not in varies:
+                    varies.append("Accept-Encoding")
+                response.headers['Vary'] = ", ".join(varies)
+                
+                response.headers['Content-Encoding'] = 'gzip'
+                response.body = compress(response.body, level)
+                if response.headers.has_key("Content-Length"):
+                    # Delete Content-Length header so finalize() recalcs it.
+                    del response.headers["Content-Length"]
+            return response
+    return HTTPError(response.request, response, 406,
+            description="identity, gzip")
